<?php

namespace Drupal\dynamic_tasks\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Component\Serialization\Exception\InvalidDataTypeException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\dynamic_tasks\Form\SettingsForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines dynamic local tasks.
 */
class DynamicLocalTasks extends DeriverBase implements ContainerDeriverInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * DynamicLocalTasks constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $configFactory) {
    $this->configFactory = $configFactory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition): array {
    if ($config = $this->configFactory->get(SettingsForm::SETTINGS)->get('tasks')) {
      try {
        foreach (array_filter(Yaml::decode($config)) as $task => $definition) {
          $this->derivatives[$task] = $definition + $base_plugin_definition;
        }
      }
      catch (InvalidDataTypeException $e) {
        watchdog_exception('dynamic_tasks', $e);
      }
    }
    return parent::getDerivativeDefinitions($base_plugin_definition);
  }

}
